using System;
using System.Collections.Generic;
using System.Text;

using Microsoft.SharePoint;
using Microsoft.SharePoint.Portal;
using Microsoft.SharePoint.Utilities;
using Microsoft.SharePoint.WebControls;

namespace SharePoint_Tool_Kit
{
    class CreateSiteWebServiceClass
    {
        /// <summary>
        /// This method creates WSS Sites.
        /// </summary>
        /// <param name="parentSiteURL">The URL of the parent site for the site that will be created</param>
        /// <param name="url">The url text for the new site</param>
        /// <param name="name">The friendly name for the new site</param>
        /// <param name="description">The description of the new site</param>
        /// <param name="localID">Local ID (1033 for English)</param>
        /// <param name="template">The template to create the new site with</param>
        /// <param name="useUniquePermissions">True of false indicating if the site should use unique permissions (true for unique perms, false to inherit perms fromt he parent site)</param>
        /// <returns>Success or Failure indicating the status of the attempt</returns>
        //Uncomment the line below this one when implementing into the Web Service you create on the MOSS Server.
        //[WebMethod]
        public string CreateWSSSite(string parentSiteURL, string url, string name, string description, int localID, string template, bool useUniquePermissions)
        {
            try
            {
                SPSite newSite = new SPSite(parentSiteURL);
                SPWeb newWeb = newSite.OpenWeb();
                newWeb.AllowUnsafeUpdates = true;
                SPWebCollection subSites = newWeb.Webs;

                SPWeb newSubWeb = subSites.Add(url, name, description, Convert.ToUInt32(localID), template, useUniquePermissions, false);

                newSubWeb.Navigation.UseShared = true;

                return "Success";

            }
            catch (System.Exception ex)
            {
                if (ex.Message == "Object reference not set to an instance of an object." && ex.StackTrace.Contains("at Microsoft.SharePoint.SPSite.get_Features()") == true)
                {

                    return "Success";
                }

                else
                {
                    return "Failure" + ex.Message + ex.StackTrace;
                }
            }
        }

        //Uncomment the line below this one when implementing into the Web Service you create on the MOSS Server.
        //[WebMethod]
        public string RegisterInSiteDirectory(string serverName, string siteDirectoryURL, string uRL, string title, string description)
        {
            try
            {
                SPSite siteDirectory = new SPSite("http://" + serverName + "/" + siteDirectoryURL);

                SPWeb siteDirectoryWeb = siteDirectory.OpenWeb();

                SPListItem newSiteListItem = siteDirectoryWeb.Lists["Sites"].Items.Add();

                newSiteListItem["URL"] = "http://" + serverName + "/" + siteDirectoryURL + "/" + uRL;
                newSiteListItem["Title"] = title;
                newSiteListItem["Description"] = description;

                newSiteListItem.Update();

                return "Success";
            }
            catch (System.Exception ex)
            {
                return "Failure";
            }
        }
    }
}
