using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Runtime.InteropServices;
using System.Text;
using System.Windows.Forms;
using System.Xml;

using Microsoft.SharePoint;
using Microsoft.SharePoint.Portal;
using Microsoft.SharePoint.Portal.Administration;
using Microsoft.SharePoint.Utilities;
using Microsoft.SharePoint.Portal.Topology;
using Microsoft.SharePoint.Administration;
using Microsoft.SharePoint.Navigation;

namespace SharePoint_Tool_Kit
{
    public partial class MainForm : Form
    {
        #region Auto scrolling functionality

        private const int WM_HSCROLL = 0x114;
        private const int WM_VSCROLL = 0x115;
        private const int SB_LINEUP = 0;
        private const int SB_LINELEFT = 0;
        private const int SB_LINEDOWN = 1;
        private const int SB_LINERIGHT = 1;
        private const int SB_PAGEUP = 2;
        private const int SB_PAGELEFT = 2;
        private const int SB_PAGEDOWN = 3;
        private const int SB_PAGERIGHT = 3;
        private const int SB_THUMBPOSITION = 4;
        private const int SB_THUMBTRACK = 5;
        private const int SB_TOP = 6;
        private const int SB_LEFT = 6;
        private const int SB_BOTTOM = 7;
        private const int SB_RIGHT = 7;
        private const int SB_ENDSCROLL = 8;

        [DllImport("user32.dll")]
        private static extern int SendMessage(IntPtr hWnd, int wMsg, IntPtr wParam,
        IntPtr lParam);

        #endregion

        #region Class level objects

        public SPWeb currentWeb;

        public SPNavigationNode currentQuickLaunchNode;

        public SPNavigationNode currentTopNavigationNode;

        private TreeUtilities treeUtilities;

        private XmlNode listTemplatesNode;

        private XmlNode listsNode;

        #endregion

        #region Constructor

        public MainForm()
        {
            InitializeComponent();

            PopulateVirtualServerList();
        }

        #endregion

        #region General Methods

        private void PopulateVirtualServerList()
        {
            SPGlobalAdmin globalAdmin = new SPGlobalAdmin();

            SPVirtualServerCollection virtualServerCollection = globalAdmin.VirtualServers;

            SendMessageToStatusTextBox("Populating virtual server list...");
            if (virtualServerCollection.Count > 0)
            {
                SendMessageToStatusTextBox(virtualServerCollection.Count.ToString() + " virtual servers found." +
                    Environment.NewLine + "List of virtual servers:" + Environment.NewLine);

            }
            else
            {
                SendMessageToStatusTextBox("No virtual servers were found on the machine." + Environment.NewLine +
                    "Please verify the name of the SharePoint Server is correct");
            }
            foreach (SPVirtualServer vs in virtualServerCollection)
            {
                virtualServerList.Items.Add(vs.Description.ToString() + "-Url-" + vs.Url.ToString() + "-Port-" + vs.Port);
                SendMessageToStatusTextBox(vs.Description.ToString() + "-Url-" + vs.Url.ToString() + "-Port-" + vs.Port);
            }
        }

        private void SendMessageToStatusTextBox(string message)
        {
            string formattedMessage = System.DateTime.Now.ToShortTimeString() + " - " + message + Environment.NewLine;
            this.statusTextBox.AppendText(formattedMessage);
            SendMessage(statusTextBox.Handle, WM_VSCROLL, (IntPtr)SB_BOTTOM, IntPtr.Zero);
        }

        #endregion

        #region Event Handlers

        private void quickLaunchNodesTreeView_MouseDown(object sender, MouseEventArgs e)
        {
            if (e.Button == MouseButtons.Right || e.Button == MouseButtons.Left)
            {
                quickLaunchNodesTreeView.SelectedNode = quickLaunchNodesTreeView.GetNodeAt(e.X, e.Y);
            }

            if (quickLaunchNodesTreeView.SelectedNode != null)
            {
                currentQuickLaunchNode = treeUtilities.GetSPQuickLaunchNavigationNodeFromTreeNode(currentWeb,
                        quickLaunchNodesTreeView.SelectedNode);
                if (currentQuickLaunchNode != null)
                {
                    PopulateQuickLaunchMenuItemMetaData();
                }
            }
        }

        private void topNavigationNodesTreeView_MouseDown(object sender, MouseEventArgs e)
        {
            if (e.Button == MouseButtons.Right || e.Button == MouseButtons.Left)
            {
                topNavigationNodesTreeView.SelectedNode = topNavigationNodesTreeView.GetNodeAt(e.X, e.Y);
            }

            if (topNavigationNodesTreeView.SelectedNode != null)
            {
                currentTopNavigationNode = treeUtilities.GetSPTopNavigationNodeFromTreeNode(currentWeb,
                        topNavigationNodesTreeView.SelectedNode);
                if (currentTopNavigationNode != null)
                {
                    PopulateTopNavigationMenuItemMetaData();
                }
            }
        }

        private void createTopLevelSiteButton_Click(object sender, EventArgs e)
        {
            SendMessageToStatusTextBox("Attempting to create top level site...");

            CreateTopLevelSite(topLevelSiteURLTextBox.Text,
                                                       topLevelSiteTitleTextBox.Text,
                                                       topLevelSiteDescriptionTextBox.Text,
                                                       Convert.ToUInt32(topLevelSiteLCIDTextBox.Text),
                                                       topLevelSiteTemplateTextBox.Text);

            RefreshSitesTree();
        }

        private void createTopLevelSiteViaWebServicesButton_Click(object sender, EventArgs e)
        {
            SendMessageToStatusTextBox("Attempting to create top level site via web services...");

            CreateTopLevelSiteViaWebServices(topLevelSiteURLTextBox.Text,
                                                       topLevelSiteTitleTextBox.Text,
                                                       topLevelSiteDescriptionTextBox.Text,
                                                       Convert.ToUInt32(topLevelSiteLCIDTextBox.Text),
                                                       topLevelSiteTemplateTextBox.Text);

            RefreshSitesTree();
        }

        private void createSubSiteButton_Click(object sender, EventArgs e)
        {
            SendMessageToStatusTextBox("Attempting to create sub site...");

            bool subSiteInheritPermissions;

            if (subSiteInheritPermissionsCheckBox.Checked == true)
            {
                subSiteInheritPermissions = true;
            }
            else
            {
                subSiteInheritPermissions = false;
            }

            CreateSubSite(subSiteParentSiteURLTextBox.Text,
                          subSiteUrlTextBox.Text,
                          subSiteTitleTextBox.Text,
                          subSiteDescriptionTextBox.Text,
                          Convert.ToUInt32(subSiteLCIDTextBox.Text),
                          subSiteTemplateTextBox.Text,
                          subSiteInheritPermissions);

            RefreshSitesTree();
        }

        private void virtualServerList_SelectedIndexChanged(object sender, EventArgs e)
        {
            treeUtilities = new TreeUtilities(this);

            treeUtilities.PopulateTreeWithSiteNodes(ref sitesTree);
        }

        private void sitesTree_MouseDown(object sender, MouseEventArgs e)
        {
            if (e.Button == MouseButtons.Right)
            {
                sitesTree.SelectedNode = sitesTree.GetNodeAt(e.X, e.Y);
            }
        }

        private void refreshButton_Click(object sender, EventArgs e)
        {
            RefreshSitesTree();
        }

        #endregion

        #region Site Related Functionality

        private void CreateTopLevelSiteViaWebServices(string uRL, string title, string description,
    uint lCID, string template)
        {
            SendMessageToStatusTextBox("Not yet implemented.  To implement this functionality take the Web Methods in the CreateSiteWebServiceClass.cs file you will find in this VS.NET Solution and add them to a Web Service you create that will be deployed on the MOSS server.  Then call the Web Methods from your Web Service.");
        }

        private void CreateTopLevelSite(string uRL, string title, string description,
            uint lCID, string template)
        {
            try
            {
                TreeNode node = treeUtilities.GetSelectedTreeNode();

                SPSite parentSite = treeUtilities.GetSPSiteFromTreeNode(node);

                SPWebCollection portalSiteWebs = parentSite.AllWebs;

                bool useUniqePermissions;

                if (inheritPermissionCheckBox.Checked == true)
                {
                    useUniqePermissions = false;
                }
                else
                {
                    useUniqePermissions = true;
                }

                if (node.Text == "Sites")
                {
                    uRL = "/sitedirectory/" + uRL;
                }

                SPWeb newWeb = portalSiteWebs.Add(uRL, title, description, lCID, template, useUniqePermissions, false);

                if (topLevelSiteUseSharedNavigationCheckBox.Checked == true)
                {
                    newWeb.Navigation.UseShared = true;
                }
                else
                {
                    newWeb.Navigation.UseShared = false;
                }


                SendMessageToStatusTextBox("Top Level Site successfully created!");
                SendMessageToStatusTextBox("Top Level Site  URL: " + newWeb.Url);
            }
            catch (System.Exception Ex)
            {
                SendMessageToStatusTextBox(Ex.Message + Environment.NewLine + Ex.StackTrace);
            }
        }

        private void CreateSubSite(string parentSiteURL, string url,
    string name, string description, uint localID, string template,
    bool useUniquePermissions)
        {
            try
            {
                SPSite newSite = new SPSite(parentSiteURL);
                SPWeb newWeb = newSite.OpenWeb();
                newWeb.AllowUnsafeUpdates = true;
                SPWebCollection subSites = newWeb.Webs;

                SPWeb newSubWeb = subSites.Add(url, name, description, Convert.ToUInt32(localID), template, useUniquePermissions, false);

                if (useUniquePermissions == true)
                {


                }

                if (subSiteUseSharedNavigationCheckBox.Checked == true)
                {
                    //Turn on Shared Navigation
                    newSubWeb.Navigation.UseShared = true;
                }
                else
                {
                    //Turn off Shared Navigation
                    newSubWeb.Navigation.UseShared = false;
                }

                newSubWeb.Update();

                SendMessageToStatusTextBox("Sub site successfully created!");
                SendMessageToStatusTextBox("Sub site URL: " + newSubWeb.Url);
            }
            catch (System.Exception ex)
            {
                if (ex.Message == "Object reference not set to an instance of an object." && ex.StackTrace.Contains("   at Microsoft.SharePoint.SPSite.get_Features()\r\n   at Microsoft.SharePoint.SPElementProvider.QueryForWorkflowDefinitions(SPWeb web)\r\n   at Microsoft.SharePoint.Workflow.SPWorkflowManager.RegisterFeatureTemplates(SPWorkflowTemplateCollection wftemplates, SPWeb web)\r\n   at Microsoft.SharePoint.SPWeb.get_WorkflowTemplates()\r\n   at Microsoft.SharePoint.Workflow.SPWorkflowAssociation.get_BaseTemplate()\r\n   at Microsoft.SharePoint.SPList.CreateWorkflowStatusColumn(SPWorkflowAssociation wa)\r\n   at Microsoft.SharePoint.Workflow.SPWorkflowAssociationCollection.AddCore(SPWorkflowAssociation wa, Guid id, SPList list, Boolean forceUtilityListCreation)\r\n   at Microsoft.SharePoint.Workflow.SPContentTypeWorkflowAssociationCollection.AddCoreCT(SPWorkflowAssociation associationTemplate)\r\n   at Microsoft.SharePoint.Workflow.SPContentTypeWorkflowAssociationCollection.UpdateOrAdd(SPWorkflowAssociation associationTemplate)\r\n   at Microsoft.SharePoint.SPContentType.CopyWorkflowAssociationsTo(SPContentType ctDst)\r\n   at Microsoft.SharePoint.SPWeb.SyncNewLists()\r\n   at Microsoft.SharePoint.SPWeb.ApplyWebTemplate(String strWebTemplate)\r\n   at Microsoft.SharePoint.SPWeb.CreateWeb(String strWebUrl, String strTitle, String strDescription, UInt32 nLCID, String strWebTemplate, Boolean bCreateUniqueSubweb, Boolean bConvertIfThere)\r\n   at Microsoft.SharePoint.SPWeb.SPWebCollectionProvider.CreateWeb(String strWebUrl, String strTitle, String strDescription, UInt32 nLCID, String strWebTemplate, Boolean bCreateUniqueSubweb, Boolean bConvertIfThere)\r\n   at Microsoft.SharePoint.SPWebCollection.Add(String strWebUrl, String strTitle, String strDescription, UInt32 nLCID, String strWebTemplate, Boolean useUniquePermissions, Boolean bConvertIfThere)\r\n") == true)
                {

                    SendMessageToStatusTextBox("Exception thrown, but the sub site was successfully created!");
                }

                else
                {
                    SendMessageToStatusTextBox(ex.Message + Environment.NewLine + ex.StackTrace);
                }
            }
        }

        private void RefreshSitesTree()
        {
            if (virtualServerList.Text.Length > 0)
            {
                sitesTree.Nodes.Clear();
                treeUtilities.PopulateTreeWithSiteNodes(ref sitesTree);
                sitesTree.Refresh();
                sitesTree.ExpandAll();
                this.sharePointServerNameTextBox.Text = virtualServerList.Text;
            }
        }

        public void DeleteSite()
        {
            RemoveFromSiteDirectory();

            this.currentWeb.Delete();

            RefreshSitesTree();

            //Output the success message to the output textbox on the form
            SendMessageToStatusTextBox("Site: " + currentWeb.Url
                    + " has been deleted.");
        }

        public void ListInSiteDirectory()
        {
            //Output the success message to the output textbox on the form
            SendMessageToStatusTextBox("Listing the site " + this.currentWeb.Name +
                    " in the Site Directory...");

            SPSite portalSite = new SPSite(this.sharePointServerNameTextBox.Text);
            SPWeb siteDirectory = portalSite.AllWebs["SiteDirectory"];
            SPList siteDirectoryList = siteDirectory.Lists["Sites"];

            //Create new item in the list 
            SPListItem item = siteDirectoryList.Items.Add();

            //Set default and required values 
            item["Title"] = this.currentWeb.Title;
            item["URL"] = this.currentWeb.Url;
            item["Description"] = this.currentWeb.Description;

            //Save the list item to the database 
            item.Update();

            //Save the item's id 
            int id = item.ID;

            //Approve the list item if moderation is enabled 
            if (siteDirectoryList.EnableModeration)
            {
                SPListItem itemT = siteDirectoryList.GetItemById(id);
                itemT.ModerationInformation.Status = SPModerationStatusType.Approved;
                //Update the item 
                itemT.Update();
            }

            //Output the success message to the output textbox on the form
            SendMessageToStatusTextBox("Site successfully listed in the Site Directory.");
        }

        public void RemoveFromSiteDirectory()
        {
            //Output the success message to the output textbox on the form
            SendMessageToStatusTextBox("Removing the site " + this.currentWeb.Name +
                    " from the Site Directory...");

            SPSite portalSite = new SPSite("http://moss-b2");
            SPWeb siteDirectory = portalSite.AllWebs["SiteDirectory"];
            SPList siteDirectoryList = siteDirectory.Lists["Sites"];

            foreach (SPListItem listItem in siteDirectoryList.Items)
            {
                if (listItem["URL"].ToString() == this.currentWeb.Url)
                {
                    siteDirectoryList.Items[listItem.ID].Delete();
                }
            }

            //Output the success message to the output textbox on the form
            SendMessageToStatusTextBox("Site successfully removed from the Site Directory.");
        }

        public void PopulateSiteMetaData()
        {
            this.siteTitleTextBox.Text = currentWeb.Title;
            this.siteURLTextBox.Text = currentWeb.Url;
            this.siteDescriptionTextBox.Text = currentWeb.Description;
            this.siteTemplateTextBox.Text = currentWeb.WebTemplate;
            this.siteConfigurationTextBox.Text = currentWeb.Configuration.ToString();
            this.siteTemplateIDTextBox.Text = currentWeb.WebTemplateId.ToString();
            this.siteGUIDTextBox.Text = currentWeb.ID.ToString();

            if (currentWeb.HasUniqueRoleAssignments == true)
            {
                this.hasUniqueRoleAssignmentsCheckBox.Checked = true;
            }
            else
            {
                this.hasUniqueRoleAssignmentsCheckBox.Checked = false;
            }

            if (currentWeb.HasUniqueRoleDefinitions == true)
            {
                this.hasUniqueRoleDefinitionsCheckBox.Checked = true;
            }
            else
            {
                this.hasUniqueRoleDefinitionsCheckBox.Checked = false;
            }

            this.featuresComboBox.Items.Clear();

            foreach (SPFeature feature in currentWeb.Features)
            {
                this.featuresComboBox.Items.Add(feature.Definition.DisplayName);
            }
            
            this.tabControl1.SelectTab("siteMetaDataTabPage");
            this.tabControl1.Refresh();
        }

        #endregion

        #region Navigation Functionality

        public void DeleteQuickLaunchItemFromQuickLaunch()
        {
            //Delete the currently selected QuickLaunch node
            currentQuickLaunchNode.Delete();
            //Update the currently selected site to commit the changes so no IIS reset is required
            currentWeb.Update();
            //Clear the TreeView that displays the QuickLaunch Menu Items
            this.quickLaunchNodesTreeView.Nodes.Clear();
            //Repopulate the TreeView that displays the QuickLaunch Menu Items
            treeUtilities.PopulateQuickLaunchTree(currentWeb);
        }

        public void DeleteTopNavigationItemFromTopNavigation()
        {
            //Delete the currently selected Top Navigation node
            currentTopNavigationNode.Delete();
            //Update the currently selected site to commit the changes so no IIS reset is required
            currentWeb.Update();
            //Clear the TreeView that displays the Top Navigation Menu Items
            this.topNavigationNodesTreeView.Nodes.Clear();
            //Repopulate the TreeView that displays the Top Navigation Menu Items
            treeUtilities.PopulateTopNavigationTree(currentWeb);
        }

        public void PopulateQuickLaunchMenuItemMetaData()
        {
            //if (currentQuickLaunchNode.Parent.Title != "Quick Launch")
            //{
            this.parentQuickLaunchMenuItemLabel.Text = currentQuickLaunchNode.Parent.Title;
            this.selectedQuickLaunchNodeIDLabel.Text = currentQuickLaunchNode.Id.ToString();
            this.selectedQuickLaunchNodeTitleLabel.Text = currentQuickLaunchNode.Title;
            this.selectedQuickLaunchNodeURLLabel.Text = currentQuickLaunchNode.Url.ToString();
            this.selectedQuickLaunchNodeIsExternalLabel.Text = currentQuickLaunchNode.IsExternal.ToString();
            //}
        }

        public void PopulateTopNavigationMenuItemMetaData()
        {
            //if (currentQuickLaunchNode.Parent.Title != "Quick Launch")
            //{
            this.parentTopNavigationMenuItemLabel.Text = currentTopNavigationNode.Parent.Title;
            this.selectedTopNavigationNodeIDLabel.Text = currentTopNavigationNode.Id.ToString();
            this.selectedTopNavigationNodeTitleLabel.Text = currentTopNavigationNode.Title;
            this.selectedTopNavigationNodeURLLabel.Text = currentTopNavigationNode.Url.ToString();
            this.selectedTopNavigationNodeIsExternalLabel.Text = currentTopNavigationNode.IsExternal.ToString();
            //}
        }

        private void addNewQuickLaunchMenuItemButton_Click(object sender, EventArgs e)
        {
            if (currentQuickLaunchNode.Parent.Title == "Quick Launch")
            {
                if (quickLaunchMenuItemLinkTextTextBox.Text.Length < 1)
                {
                    MessageBox.Show("The new QuickLaunch Menu Item must have a text value.  Please enter one and try again.", "Required Field");
                    return;
                }

                if (quickLaunchMenuItemLinkURLTextBox.Text.Length < 1)
                {
                    MessageBox.Show("The new QuickLaunch Menu Item must have a URL value.  Please enter one and try again.", "Required Field");
                    return;
                }

                SPNavigationNode newQuickLaunchNode;

                //Create the SPNavigationNode object
                if (externalQuickLaunchMenuItemCheckBox.Checked == true)
                {
                    newQuickLaunchNode = new SPNavigationNode(quickLaunchMenuItemLinkTextTextBox.Text,
                                        quickLaunchMenuItemLinkURLTextBox.Text, true);
                }
                else
                {
                    newQuickLaunchNode = new SPNavigationNode(quickLaunchMenuItemLinkTextTextBox.Text,
                                        quickLaunchMenuItemLinkURLTextBox.Text, false);
                }
                //Add the new SPNavigationNode to the currently selected QuickLaunch node
                currentQuickLaunchNode.Children.AddAsFirst(newQuickLaunchNode);

                //Update the currently selected QuickLaunch node to commit the changes
                currentQuickLaunchNode.Update();

                //Update the currently selected site to commit the changes so no IIS reset is required
                currentWeb.Update();

                //Clear the TreeView that displays the QuickLaunch Menu Items
                this.quickLaunchNodesTreeView.Nodes.Clear();
                //Repopulate the TreeView that displays the QuickLaunch Menu Items
                treeUtilities.PopulateQuickLaunchTree(currentWeb);
            }
            else
            {
                MessageBox.Show("QuickLaunch Menu Items can not be created more than one level deep.  Select a different parent site and try again.", "QuickLaunch Menu Limitation");
                return;
            }
        }

        private void addNewTopNavigationMenuItemButton_Click(object sender, EventArgs e)
        {
            if (topNavigationMenuItemLinkTextTextBox.Text.Length < 1)
            {
                MessageBox.Show("The new Top Navigation Menu Item must have a text value.  Please enter one and try again.", "Required Field");
                return;
            }

            if (topNavigationMenuItemLinkURLTextBox.Text.Length < 1)
            {
                MessageBox.Show("The new Top Navigation Menu Item must have a URL value.  Please enter one and try again.", "Required Field");
                return;
            }

            SPNavigationNode newTopNavigationNode;

            //Create the SPNavigationNode object
            if (externalTopNavigationMenuItemCheckBox.Checked == true)
            {
                newTopNavigationNode = new SPNavigationNode(topNavigationMenuItemLinkTextTextBox.Text,
                                    topNavigationMenuItemLinkURLTextBox.Text, true);
            }
            else
            {
                newTopNavigationNode = new SPNavigationNode(topNavigationMenuItemLinkTextTextBox.Text,
                                    topNavigationMenuItemLinkURLTextBox.Text, false);
            }

            //Determine if one of the Top Navigation Nodes has been selected
            if (currentTopNavigationNode == null)
            {
                //Create a SPWeb object for the currently selected web
                SPWeb topNavigationWeb = currentWeb;

                //Create a collection of the Top Navigation Nodes
                SPNavigationNodeCollection topNavigationNodes = topNavigationWeb.Navigation.TopNavigationBar;

                //Add the new SPNavigationNode to the root
                topNavigationNodes.AddAsFirst(newTopNavigationNode);
            }
            else
            {
                //Add the new SPNavigationNode to the currently selected Top Navigation node
                currentTopNavigationNode.Children.AddAsFirst(newTopNavigationNode);

                //Update the currently selected Top Navigation node to commit the changes
                currentTopNavigationNode.Update();
            }

            //Update the currently selected site to commit the changes so no IIS reset is required
            currentWeb.Update();

            //Clear the TreeView that displays the Top Navigation Menu Items
            this.topNavigationNodesTreeView.Nodes.Clear();
            //Repopulate the TreeView that displays the Top Navigation Menu Items
            treeUtilities.PopulateTopNavigationTree(currentWeb);
        }

        private void externalQuickLaunchMenuItemCheckBox_CheckedChanged(object sender, EventArgs e)
        {
            string URL = quickLaunchMenuItemLinkURLTextBox.Text;

            if (this.externalQuickLaunchMenuItemCheckBox.Checked == true)
            {
                quickLaunchMenuItemLinkURLTextBox.Text = "http://" + URL;
            }
            else
            {
                quickLaunchMenuItemLinkURLTextBox.Text = URL.Replace("http://", ""); ;
            }
        }

        private void externalTopNavigationMenuItemCheckBox_CheckedChanged(object sender, EventArgs e)
        {
            string URL = topNavigationMenuItemLinkURLTextBox.Text;

            if (this.externalTopNavigationMenuItemCheckBox.Checked == true)
            {
                topNavigationMenuItemLinkURLTextBox.Text = "http://" + URL;
            }
            else
            {
                topNavigationMenuItemLinkURLTextBox.Text = URL.Replace("http://", ""); ;
            }
        }

        #endregion

        #region User Management Functionality

        private void addUserViaWebServicesButton_Click(object sender, EventArgs e)
        {
            //Output the status message to the output textbox on the form
            SendMessageToStatusTextBox("Adding user...");

            WSSUserGroupService.UserGroup wssUserGroupService = new SharePoint_Tool_Kit.WSSUserGroupService.UserGroup();
            wssUserGroupService.Credentials = System.Net.CredentialCache.DefaultCredentials;

            string webServiceSiteSpecificURL = this.sharePointServerNameTextBox.Text;

            TreeNode node = treeUtilities.GetSelectedTreeNode();

            webServiceSiteSpecificURL = webServiceSiteSpecificURL + currentWeb.ServerRelativeUrl + "/_vti_bin/UserGroup.asmx";

            wssUserGroupService.Url = webServiceSiteSpecificURL;

            wssUserGroupService.AddUserToGroup(this.groupComboBox.Text, this.displayNameTextBox.Text,
                this.domainTextBox.Text + "\\" + this.userNameTextBox.Text, this.emailAddressTextBox.Text, this.notesTextBox.Text);

            //Output the status message to the output textbox on the form
            SendMessageToStatusTextBox("User was successfully added to the " + this.groupComboBox.Text +
                    " group.");

            this.PopulateUsersListBox(this.groupComboBox.Text);
        }

        private void removeUserViaWebServicesButton_Click(object sender, EventArgs e)
        {

            if (usersListBox.SelectedItem == null)
            {
                MessageBox.Show("Please select a user and try again.", "No user specified", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                return;
            }

            //Output the status message to the output textbox on the form
            SendMessageToStatusTextBox("Removing user...");

            WSSUserGroupService.UserGroup wssUserGroupService = new SharePoint_Tool_Kit.WSSUserGroupService.UserGroup();
            wssUserGroupService.Credentials = System.Net.CredentialCache.DefaultCredentials;

            string webServiceSiteSpecificURL = this.sharePointServerNameTextBox.Text;

            TreeNode node = treeUtilities.GetSelectedTreeNode();

            webServiceSiteSpecificURL = webServiceSiteSpecificURL + currentWeb.ServerRelativeUrl + "/_vti_bin/UserGroup.asmx";

            wssUserGroupService.Url = webServiceSiteSpecificURL;

            wssUserGroupService.RemoveUserFromGroup(this.groupComboBox.Text, this.usersListBox.SelectedItem.ToString());

            //Output the status message to the output textbox on the form
            SendMessageToStatusTextBox("User was successfully removed from the " + this.groupComboBox.Text +
                    " group.");

            this.PopulateUsersListBox(this.groupComboBox.Text);
        }

        private void addUserViaObjectModelButton_Click(object sender, EventArgs e)
        {
            //Output the status message to the output textbox on the form
            SendMessageToStatusTextBox("Adding user...");

            //Create a SPGroup object for the group currently selected
            SPGroup group = currentWeb.Groups[this.groupComboBox.Text];

            //Add a new user to the group
            group.AddUser(this.domainTextBox.Text + "\\" + userNameTextBox.Text,
                              emailAddressTextBox.Text,
                              displayNameTextBox.Text,
                              notesTextBox.Text);

            //Update the group
            group.Update();

            //Update the SPWeb
            currentWeb.Update();

            //Output the status message to the output textbox on the form
            SendMessageToStatusTextBox("User was successfully added to the " + this.groupComboBox.Text +
                    " group.");

            this.PopulateUsersListBox(this.groupComboBox.Text);
        }

        public void PopulateUserGroupsForCurrentSite()
        {
            this.groupComboBox.Items.Clear();

            //Create a collection of groups for the current site
            SPGroupCollection groups = currentWeb.Groups;

            //Add each Group to the groupComboBox
            foreach (SPGroup group in groups)
            {
                this.groupComboBox.Items.Add(group.Name);
            }
        }

        private void groupComboBox_SelectedIndexChanged(object sender, EventArgs e)
        {
            this.PopulateUsersListBox(groupComboBox.Text);
        }

        private void PopulateUsersListBox(string groupName)
        {
            this.usersListBox.Items.Clear();

            //Create a collection of all the users for a given Site Group
            SPUserCollection userCollection = currentWeb.Groups[groupName].Users;

            //Add each user in the collection to the usersListBox
            foreach (SPUser user in userCollection)
            {
                this.usersListBox.Items.Add(user.LoginName);
            }
        }

        private void removeUserButton_Click(object sender, EventArgs e)
        {
            if (usersListBox.SelectedItem == null)
            {
                MessageBox.Show("Please select a user and try again.", "No user specified", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                return;
            }

            //Create a SPGroup object for the currently select Group
            SPGroup group = currentWeb.Groups[this.groupComboBox.Text];

            //Create a SPUser object for the currently selected User
            SPUser user = group.Users[usersListBox.SelectedItem.ToString()];

            //Remove the user from the Group
            group.RemoveUser(user);

            //Update the group
            group.Update();

            //Update the SPWeb
            currentWeb.Update();

            //Output the status message to the output textbox on the form
            SendMessageToStatusTextBox("User was successfully removed from the " + this.groupComboBox.Text +
                    " group.");

            this.PopulateUsersListBox(this.groupComboBox.Text);
        }

        #endregion

        #region Lists Functionality

        private void ReturnAvailableListTemplatesButton_Click(object sender, EventArgs e)
        {
            try
            {
                SendMessageToStatusTextBox("Retrieving List Templates");

                //Create a variable representing the URL to access the Webs web service
                //for the given site
                string webServiceSiteSpecificURL = this.sharePointServerNameTextBox.Text;

                string websWSUrl = webServiceSiteSpecificURL + currentWeb.ServerRelativeUrl +
                    "/_vti_bin/Webs.asmx";
                //Create a new object representing the Webs service
                WebsService.Webs websSvc = new SharePoint_Tool_Kit.WebsService.Webs();
                //Set the credentials
                websSvc.Credentials = System.Net.CredentialCache.DefaultCredentials;
                //Set the URL
                websSvc.Url = websWSUrl;
                //Return the List Templates available on the given site
                listTemplatesNode = websSvc.GetListTemplates();
                //Create an XmlNodeList to hold theList Templates
                XmlNodeList listTemplates = listTemplatesNode.ChildNodes;
                //Create an ArrayList to bind the list of List Templates to the ComboBox
                ArrayList templates = new ArrayList();
                //Loop through the XMLNodes in the listTemplates XmlNodeList
                foreach (XmlNode ltNode in listTemplates)
                {
                    //Add the display name for each List Template to the ArrayList
                    templates.Add(ltNode.Attributes["DisplayName"].Value);
                }
                //Clear the datasource of the ComboBox
                listTemplatesComboBox.DataSource = null;
                //Clear the items in the ComboBox
                listTemplatesComboBox.Items.Clear();
                //Set the ComboBox datasource to use the templates ArrayList
                listTemplatesComboBox.DataSource = templates;
            }
            catch (Exception ex)
            {
                SendMessageToStatusTextBox("Failed to retrieve List Templates: " +
                    ex.Message);
                this.createListButton.Enabled = false;
                return;
            }

            SendMessageToStatusTextBox("Retrieved List Templates");

            this.createListButton.Enabled = true;
        }

        private void ReturnAvailableListsButton_Click(object sender, EventArgs e)
        {
            try
            {
                SendMessageToStatusTextBox("Retrieving Lists");

                //Create a new object representing the Webs service
                ListsService.Lists listsSvc = new SharePoint_Tool_Kit.ListsService.Lists();
                //Create the URL to access the Webs web service for the given site
                string webServiceSiteSpecificURL = this.sharePointServerNameTextBox.Text;
                string listWSUri = webServiceSiteSpecificURL + currentWeb.ServerRelativeUrl + "/_vti_bin/Lists.asmx";
                //Set the credentials
                listsSvc.Credentials = System.Net.CredentialCache.DefaultCredentials;
                //Set the URL
                listsSvc.Url = listWSUri;
                //Return the Lists available on the given site
                listsNode = listsSvc.GetListCollection();
                //Create an XmlNodeList to hold the Lists
                XmlNodeList lists = listsNode.ChildNodes;
                //Create an ArrayList to bind the list of Lists to a ComboBox
                ArrayList arrayOfLists = new ArrayList();
                //Loop through the XMLNodes in the listTemplates XmlNodeList
                foreach (XmlNode ltNode in lists)
                {
                    //Add the display name for each List Template to the ArrayList
                    arrayOfLists.Add(ltNode.Attributes["Title"].Value);
                }
                //Clear the datasource of the ComboBox
                listsComboBox.DataSource = null;
                //Clear the items in the ComboBox
                listsComboBox.Items.Clear();
                //Set the ComboBox datasource to use the lists ArrayList
                listsComboBox.DataSource = arrayOfLists;
            }
            catch (Exception ex)
            {
                SendMessageToStatusTextBox("Failed to retrieve Lists: " +
                    ex.Message);
                return;
            }

            SendMessageToStatusTextBox("Retrieved Lists");
        }

        private void createListButton_Click(object sender, EventArgs e)
        {
            try
            {
                SendMessageToStatusTextBox("Attempting to create List " + newListNameTextBox.Text);

                //Create a new object representing the Webs service
                ListsService.Lists listsSvc = new SharePoint_Tool_Kit.ListsService.Lists();
                //Create the URL to access the Webs web service for the given site
                string webServiceSiteSpecificURL = this.sharePointServerNameTextBox.Text;
                string listWSUri = webServiceSiteSpecificURL + currentWeb.ServerRelativeUrl + "/_vti_bin/Lists.asmx";
                //Set the credentials
                listsSvc.Credentials = System.Net.CredentialCache.DefaultCredentials;
                //Set the URL
                listsSvc.Url = listWSUri;
                //Find the ID of the selected item in the ComboBox
                int selectedTemplateId = listTemplatesComboBox.SelectedIndex;
                //Create a variable to hold the name of the selected List Template
                string selDisplayName = listTemplatesComboBox.Items[selectedTemplateId].ToString();
                //Create an XMLNodeList to hold the List Templates already returned from the given WSS site
                XmlNodeList ltemps = listTemplatesNode.ChildNodes;
                //Create an XmlNode to represent the node we are looking for
                XmlNode found = null;
                //Loop through the List Templates returned for the given WSS site
                foreach (XmlNode lt in ltemps)
                {
                    //Create a variable to hold the display name of the List Template
                    string dName = lt.Attributes["DisplayName"].Value;
                    //Determine if the List Template matches the selected one in the ComboBox
                    if (dName.Equals(selDisplayName))
                    {
                        //Set the found node equal to the current node in the list
                        found = lt;
                        break;
                    }
                }

                //If a match was found then continue creating the new list
                if (found != null)
                {
                    //Return the Type of the List Template
                    string listType = found.Attributes["Type"].Value;
                    //Use the Lists web service to create a new list
                    listsSvc.AddList(newListNameTextBox.Text, newListDescriptionTextBox.Text, Int32.Parse(listType));
                }

                SendMessageToStatusTextBox("Created List " + newListNameTextBox.Text);

                this.createListButton.Enabled = false;
            }
            catch (System.Web.Services.Protocols.SoapException soapException)
            {
                SendMessageToStatusTextBox("Failed to Create List " + newListNameTextBox.Text + " " + soapException.InnerException);
            }
            catch (Exception ex)
            {
                SendMessageToStatusTextBox("Failed to Create List " + newListNameTextBox.Text + " " + ex.Message);
            }
        }

        #endregion

        #region Feature Functionality

        public Guid ReturnFeatureGuid(string FeatureDisplayName)
        {
            Guid featureGuid = new Guid();

            //Look to see if the Feature has been activated
            foreach (SPFeature feature in currentWeb.Features)
            {
                if (feature.Definition.DisplayName == FeatureDisplayName)
                {
                    featureGuid = new Guid(feature.DefinitionId.ToString());
                    break;
                }
            }

            return featureGuid;
        }

        public void ActivateFeature(Guid featureID)
        {
            bool isAlreadyActivated = false;

            //Look to see if the Feature has been activated
            foreach (SPFeature feature in currentWeb.Features)
            {
                if (feature.Definition.Id == featureID)
                {
                    isAlreadyActivated = true;
                }
            }

            //If the Feature has not been activated, then activate it
            if (isAlreadyActivated == false)
            {
                currentWeb.Features.Add(featureID);
            }
        }

        public void DeactivateFeature(Guid featureID)
        {
            bool isAlreadyActivated = false;

            //Look to see if the Feature has been activated
            foreach (SPFeature feature in currentWeb.Features)
            {
                if (feature.Definition.Id == featureID)
                {
                    isAlreadyActivated = true;
                }
            }

            //If the Feature has been activated, then deactivate it
            if (isAlreadyActivated == true)
            {
                currentWeb.Features.Remove(featureID);
            }
        }

        #endregion

        private void DeactivateFeatureButton_Click(object sender, EventArgs e)
        {
            Guid featureGuid = ReturnFeatureGuid(this.featuresComboBox.Text);

            DeactivateFeature(featureGuid);

            SendMessageToStatusTextBox("The " + featuresComboBox.Text + " Feature has been deactivated.");

            this.featuresComboBox.Items.Clear();

            foreach (SPFeature feature in currentWeb.Features)
            {
                this.featuresComboBox.Items.Add(feature.Definition.DisplayName);
            }
        }

        private void InstallFeatureButton_Click(object sender, EventArgs e)
        {
            Guid featureGuid = new Guid(featureGuidTextBox.Text);

            ActivateFeature(featureGuid);

            SendMessageToStatusTextBox("The " + featuresComboBox.Text + " Feature has been installed and activated.");

            this.featuresComboBox.Items.Clear();

            foreach (SPFeature feature in currentWeb.Features)
            {
                this.featuresComboBox.Items.Add(feature.Definition.DisplayName);
            }
        }
    }
}