using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;

using Microsoft.SharePoint;
using Microsoft.SharePoint.Portal;
using Microsoft.SharePoint.Portal.Administration;
using Microsoft.SharePoint.Utilities;
using Microsoft.SharePoint.Portal.Topology;
using Microsoft.SharePoint.Administration;
using Microsoft.SharePoint.Navigation;

namespace SharePoint_Tool_Kit
{
    public class TreeUtilities
    {
        private MainForm mainForm;

        int count;

        TreeView sitesTree;

        private SPNavigationNode returnNode;

        public TreeUtilities(MainForm refMainForm)
        {
            this.mainForm = refMainForm;
        }

        public void PopulateTreeWithSiteNodes(ref TreeView refSitesTree)
        {
            this.mainForm.countValue.Text = "Loading...";
            this.mainForm.countValue.Refresh();

            count = 0;

            this.sitesTree = refSitesTree;

            this.sitesTree.Nodes.Clear();

            //Create a SPGlobalAdmin object to access the virtual servers
            SPGlobalAdmin globalAdmin = new SPGlobalAdmin();

            //Create a collection of the virtual servers
            SPVirtualServerCollection virtualServerCollection = globalAdmin.VirtualServers;

            //Create a site collection to hold the top level sites in the virtual server
            SPSiteCollection topLevelSites = null;

            //Loop through the virtual server collection and find the top level sites in the virtual server
            //associated with the selected virtual server
            foreach (SPVirtualServer vs in virtualServerCollection)
            {
                if (this.mainForm.virtualServerList.Text.Contains(vs.Description + "-Url-" + vs.Url + "-Port-" + vs.Port))
                {
                    topLevelSites = vs.Sites;
                    this.mainForm.sharePointServerNameTextBox.Text = vs.Url.ToString();
                    break;
                }
            }

            if (topLevelSites != null)
            {
                //Populate the tree with the top level site collections
                foreach (SPSite topLevelSite in topLevelSites)
                {
                    SPWeb topLevelWeb = topLevelSite.OpenWeb();
                    TreeNode topLevelWebNode = new TreeNode(topLevelWeb.Title.ToString());
                    topLevelWebNode.Tag = topLevelSite.ID.ToString() + "~~" + topLevelSite.Url; ;
                    this.sitesTree.Nodes.Add(topLevelWebNode);

                    AddContextMenu(topLevelWebNode);
                }
            }

            foreach (TreeNode node in sitesTree.Nodes)
            {
                Guid topLevelSiteGuid = new Guid(GetGuidFromNodeTag(node.Tag.ToString()));
                SPSite topLevelSiteChild = new SPSite(topLevelSiteGuid);
                SPWeb topLevelWebChild = topLevelSiteChild.OpenWeb();

                AddChildWebs(topLevelWebChild, node);
            }

            this.mainForm.countValue.Text = count.ToString();
        }

        private void AddChildWebs(SPWeb parentWeb, TreeNode parentNode)
        {
            foreach (SPWeb web in parentWeb.Webs)
            {
                TreeNode childNode = CreateNode(web, parentNode);

                AddContextMenu(childNode);

                if (web.Webs.Count > 0)
                {
                    AddChildWebs(web, childNode);
                }
            }
        }

        private TreeNode CreateNode(SPWeb web, TreeNode parentNode)
        {

            TreeNode childNode = new TreeNode(web.Title);
            childNode.Tag = web.ID.ToString() + "~~" + web.Url;
            parentNode.Nodes.Add(childNode);
            return childNode;
        }

        private void AddContextMenu(TreeNode node)
        {
            count++;

            ContextMenuStrip menuStrip = new ContextMenuStrip();

            bool enableTopLevelSiteCreation = false;

            //Check to see if we are on the top level of the portal
            if (GetSPWebFromTreeNode(node).ParentWeb == null)
            {
                enableTopLevelSiteCreation = true;
            }

            //Check to see if we are in the sites directory
            if (node.Text == "Sites")
            {
                enableTopLevelSiteCreation = true;
            }

            if (enableTopLevelSiteCreation)
            {
                ToolStripItem createTopLevelSiteItem = new ToolStripMenuItem("Add Top Level Site");
                createTopLevelSiteItem.Click += new EventHandler(createTopLevelSiteItem_Click);
                menuStrip.Items.Add(createTopLevelSiteItem);

                ToolStripItem createTopLevelSiteViaWebServicesItem = new ToolStripMenuItem("Add Top Level Site Via Web Services");
                createTopLevelSiteViaWebServicesItem.Click += new EventHandler(createTopLevelSiteViaWebServicesItem_Click);
                menuStrip.Items.Add(createTopLevelSiteViaWebServicesItem);
            }
            else
            {
                ToolStripItem createSubSiteItem = new ToolStripMenuItem("Add Sub Site");
                createSubSiteItem.Click += new EventHandler(createSubSiteItem_Click);
                menuStrip.Items.Add(createSubSiteItem);
            }

            //Create the menu item to display the meta data
            ToolStripItem siteMetaDataItem = new ToolStripMenuItem("Query Meta Data");
            siteMetaDataItem.Click += new EventHandler(siteMetaDataItem_Click);
            menuStrip.Items.Add(siteMetaDataItem);

            //Create the menu item to query / edit QuickLaunch Menu Items
            ToolStripItem queryEditQuickLaunchItems = new ToolStripMenuItem("Query / Edit QuickLaunch Menu Items");
            queryEditQuickLaunchItems.Click += new EventHandler(queryEditQuickLaunchItems_Click);
            menuStrip.Items.Add(queryEditQuickLaunchItems);

            //Create the menu item to add a Top Navigation Menu Item
            ToolStripItem addTopNavigationItem = new ToolStripMenuItem("Query / Edit Top Navigation Menu Items");
            addTopNavigationItem.Click += new EventHandler(addTopNavigationItem_Click);
            menuStrip.Items.Add(addTopNavigationItem);

            //Create the menu item to delete a site
            ToolStripItem deleteSiteItem = new ToolStripMenuItem("Delete Site");
            deleteSiteItem.Click += new EventHandler(deleteSiteItem_Click);
            menuStrip.Items.Add(deleteSiteItem);

            //Create the menu item to list a site in the site directory
            ToolStripItem listInSiteDirectoryItem = new ToolStripMenuItem("List In Site Directory");
            listInSiteDirectoryItem.Click += new EventHandler(listInSiteDirectoryItem_Click);
            menuStrip.Items.Add(listInSiteDirectoryItem);

            //Create the menu item to edit user permissions
            ToolStripItem editUserPermissionsItem = new ToolStripMenuItem("Edit User Permissions");
            editUserPermissionsItem.Click += new EventHandler(editUserPermissionsItem_Click);
            menuStrip.Items.Add(editUserPermissionsItem);

            //Create the menu item to create Lists
            ToolStripItem createListsItem = new ToolStripMenuItem("Create / Query Lists");
            createListsItem.Click += new EventHandler(createListsItem_Click);
            menuStrip.Items.Add(createListsItem);

            //Add the menu strip to the node
            node.ContextMenuStrip = menuStrip;
        }

        void listInSiteDirectoryItem_Click(object sender, EventArgs e)
        {
            this.mainForm.currentWeb = GetSPWebFromTreeNode(GetSelectedTreeNode());
            this.mainForm.ListInSiteDirectory();
        }

        void editUserPermissionsItem_Click(object sender, EventArgs e)
        {
            this.mainForm.currentWeb = GetSPWebFromTreeNode(GetSelectedTreeNode());
            this.mainForm.tabControl1.SelectTab("usersTabPage");
            this.mainForm.tabControl1.Refresh();
            this.mainForm.userNameTextBox.Text = "";
            this.mainForm.usersListBox.Items.Clear();
            this.mainForm.displayNameTextBox.Text = "";
            this.mainForm.notesTextBox.Text = "";
            this.mainForm.emailAddressTextBox.Text = "";
            this.mainForm.domainTextBox.Text = "";
            this.mainForm.groupComboBox.Items.Clear();
            this.mainForm.groupComboBox.Text = "";
            this.mainForm.PopulateUserGroupsForCurrentSite();
        }

        void createListsItem_Click(object sender, EventArgs e)
        {
            this.mainForm.currentWeb = GetSPWebFromTreeNode(GetSelectedTreeNode());
            this.mainForm.tabControl1.SelectTab("listsTabPage");
            this.mainForm.tabControl1.Refresh();
            this.mainForm.newListNameTextBox.Text = "";
            this.mainForm.newListDescriptionTextBox.Text = "";
            this.mainForm.listTemplatesComboBox.Items.Clear();
            this.mainForm.ReturnAvailableListTemplatesButton.Enabled = true;
        }

        public void PopulateQuickLaunchTree(SPWeb currentWeb)
        {
            try
            {
                this.mainForm.quickLaunchNodesTreeView.Nodes.Clear();

                SPNavigationNodeCollection quickLaunchNodes = currentWeb.Navigation.QuickLaunch;

                SPNavigationNode rootQuickLaunchNode = quickLaunchNodes.Parent;
                TreeNode rootQuickLaunchTreeNode = new TreeNode(rootQuickLaunchNode.Title);
                rootQuickLaunchTreeNode.Tag = rootQuickLaunchNode.Id.ToString() + "~~" + rootQuickLaunchNode.Url;
                this.mainForm.quickLaunchNodesTreeView.Nodes.Add(rootQuickLaunchTreeNode);

                foreach (SPNavigationNode quickLaunchNode in quickLaunchNodes)
                {
                    TreeNode topLevelQuickLaunchNode = new TreeNode(quickLaunchNode.Title);
                    topLevelQuickLaunchNode.Tag = quickLaunchNode.Id.ToString() + "~~" + quickLaunchNode.Url;
                    rootQuickLaunchTreeNode.Nodes.Add(topLevelQuickLaunchNode);

                    AddQuickLaunchNodeContextMenu(topLevelQuickLaunchNode);

                    AddChildQuickLaunchNodes(quickLaunchNode, topLevelQuickLaunchNode);
                }
            }
            catch (System.Exception Ex)
            {
                string ExceptionMessage = Ex.Message;
            }
            finally
            {
                this.mainForm.addNewQuickLaunchMenuItemButton.Enabled = false;
            }
        }

        public void PopulateTopNavigationTree(SPWeb currentWeb)
        {
            try
            {
                this.mainForm.topNavigationNodesTreeView.Nodes.Clear();

                SPNavigationNodeCollection topNavigationNodes = currentWeb.Navigation.TopNavigationBar;

                SPNavigationNode rootTopNavigationNode = topNavigationNodes.Parent;
                TreeNode rootTopNavigationTreeNode = new TreeNode(rootTopNavigationNode.Title);
                rootTopNavigationTreeNode.Tag = rootTopNavigationNode.Id.ToString() + "~~" + rootTopNavigationNode.Url;
                this.mainForm.topNavigationNodesTreeView.Nodes.Add(rootTopNavigationTreeNode);

                foreach (SPNavigationNode topNavigationNode in topNavigationNodes)
                {
                    TreeNode topLevelTopNavigationNode = new TreeNode(topNavigationNode.Title);
                    topLevelTopNavigationNode.Tag = topNavigationNode.Id.ToString() + "~~" + topNavigationNode.Url;
                    rootTopNavigationTreeNode.Nodes.Add(topLevelTopNavigationNode);

                    AddTopNavigationNodeContextMenu(topLevelTopNavigationNode);

                    AddChildTopNavigationNodes(topNavigationNode, topLevelTopNavigationNode);
                }
            }
            catch (System.Exception Ex)
            {
                string ExceptionMessage = Ex.Message;
            }
            finally
            {
                this.mainForm.addNewQuickLaunchMenuItemButton.Enabled = false;
            }

        }

        private void AddChildQuickLaunchNodes(SPNavigationNode quickLaunchNode, TreeNode parentNode)
        {
            if (quickLaunchNode.Children.Count > 0)
            {
                foreach (SPNavigationNode childQuickLaunchNode in quickLaunchNode.Children)
                {
                    TreeNode childNode = new TreeNode(childQuickLaunchNode.Title);
                    childNode.Tag = childQuickLaunchNode.Id.ToString() + "~~" + childQuickLaunchNode.Url;
                    parentNode.Nodes.Add(childNode);

                    AddQuickLaunchNodeContextMenu(childNode);

                    AddChildQuickLaunchNodes(childQuickLaunchNode, childNode);
                }
            }
        }

        private void AddQuickLaunchNodeContextMenu(TreeNode node)
        {
            ContextMenuStrip menuStrip = new ContextMenuStrip();

            ToolStripItem deleteQuickLaunchNodeItem = new ToolStripMenuItem("Delete Quick Launch Item");
            deleteQuickLaunchNodeItem.Click += new EventHandler(deleteQuickLaunchNodeItem_Click);
            menuStrip.Items.Add(deleteQuickLaunchNodeItem);

            ToolStripItem addQuickLaunchNodeItem = new ToolStripMenuItem("Add Quick Launch Item");
            addQuickLaunchNodeItem.Click += new EventHandler(addQuickLaunchNodeItem_Click);
            menuStrip.Items.Add(addQuickLaunchNodeItem);

            //ToolStripItem createSubSiteItem = new ToolStripMenuItem("Add Sub Site");
            //createSubSiteItem.Click += new EventHandler(createSubSiteItem_Click);
            //menuStrip.Items.Add(createSubSiteItem);

            //Add the menu strip to the node
            node.ContextMenuStrip = menuStrip;
        }

        private void AddChildTopNavigationNodes(SPNavigationNode topNavigationNode, TreeNode parentNode)
        {
            if (topNavigationNode.Children.Count > 0)
            {
                foreach (SPNavigationNode childTopNavigationNode in topNavigationNode.Children)
                {
                    TreeNode childNode = new TreeNode(childTopNavigationNode.Title);
                    childNode.Tag = childTopNavigationNode.Id.ToString() + "~~" + childTopNavigationNode.Url;
                    parentNode.Nodes.Add(childNode);

                    AddTopNavigationNodeContextMenu(childNode);

                    AddChildTopNavigationNodes(childTopNavigationNode, childNode);
                }
            }
        }

        private void AddTopNavigationNodeContextMenu(TreeNode node)
        {
            ContextMenuStrip menuStrip = new ContextMenuStrip();

            ToolStripItem deleteTopNavigationNodeItem = new ToolStripMenuItem("Delete Top Navigation Item");
            deleteTopNavigationNodeItem.Click += new EventHandler(deleteTopNavigationNodeItem_Click);
            menuStrip.Items.Add(deleteTopNavigationNodeItem);

            ToolStripItem addTopNavigationNodeItem = new ToolStripMenuItem("Add Top Navigation Item");
            addTopNavigationNodeItem.Click += new EventHandler(addTopNavigationNodeItem_Click);
            menuStrip.Items.Add(addTopNavigationNodeItem);

            //Add the menu strip to the node
            node.ContextMenuStrip = menuStrip;
        }

        #region QuickLaunch Tree Node Event Handlers

        void deleteQuickLaunchNodeItem_Click(object sender, EventArgs e)
        {
            TreeNode selectedQuickLaunchTreeNode = GetSelectedQuickLaunchTreeNode();

            SPNavigationNode selectedQuickLaunchNode = GetSPQuickLaunchNavigationNodeFromTreeNode(this.mainForm.currentWeb, selectedQuickLaunchTreeNode);
            this.mainForm.currentQuickLaunchNode = selectedQuickLaunchNode;

            DialogResult result;

            result = MessageBox.Show("Are you sure you want to delete the " + selectedQuickLaunchNode.Title + " QuickLaunch Menu Item?",
                "QuickLaunch Menu Item Deletion Confirmation", MessageBoxButtons.OKCancel);
            if (result == DialogResult.OK)
            {
                this.mainForm.DeleteQuickLaunchItemFromQuickLaunch();
            }
        }

        void addQuickLaunchNodeItem_Click(object sender, EventArgs e)
        {
            TreeNode selectedQuickLaunchTreeNode = GetSelectedQuickLaunchTreeNode();

            SPNavigationNode selectedQuickLaunchNode = GetSPQuickLaunchNavigationNodeFromTreeNode(this.mainForm.currentWeb, selectedQuickLaunchTreeNode);
            this.mainForm.currentQuickLaunchNode = selectedQuickLaunchNode;
            this.mainForm.addNewQuickLaunchMenuItemButton.Enabled = true;
        }

        #endregion

        #region TopNavigation Tree Node Event Handlers

        void deleteTopNavigationNodeItem_Click(object sender, EventArgs e)
        {
            TreeNode selectedTopNavigationTreeNode = GetSelectedTopNavigationTreeNode();

            SPNavigationNode selectedTopNavigationNode = GetSPTopNavigationNodeFromTreeNode(this.mainForm.currentWeb, selectedTopNavigationTreeNode);
            this.mainForm.currentTopNavigationNode = selectedTopNavigationNode;

            DialogResult result;

            result = MessageBox.Show("Are you sure you want to delete the " + selectedTopNavigationNode.Title + " TopNavigation Menu Item?",
                "Top Navigation Menu Item Deletion Confirmation", MessageBoxButtons.OKCancel);
            if (result == DialogResult.OK)
            {
                this.mainForm.DeleteTopNavigationItemFromTopNavigation();
            }
        }

        void addTopNavigationNodeItem_Click(object sender, EventArgs e)
        {
            TreeNode selectedTopNavigationTreeNode = GetSelectedTopNavigationTreeNode();

            SPNavigationNode selectedTopNavigationNode = GetSPTopNavigationNodeFromTreeNode(this.mainForm.currentWeb, selectedTopNavigationTreeNode);
            this.mainForm.currentTopNavigationNode = selectedTopNavigationNode;
            this.mainForm.addNewTopNavigationMenuItemButton.Enabled = true;
        }

        #endregion

        #region Event Handers for ToolStripMenuItems

        private void siteMetaDataItem_Click(object sender, EventArgs e)
        {
            this.mainForm.currentWeb = GetSPWebFromTreeNode(GetSelectedTreeNode());
            this.mainForm.PopulateSiteMetaData();
        }

        private void createSubSiteItem_Click(object sender, EventArgs e)
        {
            this.mainForm.currentWeb = GetSPWebFromTreeNode(GetSelectedTreeNode());
            this.mainForm.subSiteParentSiteURLTextBox.Text = this.mainForm.currentWeb.Url;
            this.mainForm.tabControl1.SelectTab("subSitesTabPage");
            this.mainForm.tabControl1.Refresh();
        }

        private void createTopLevelSiteItem_Click(object sender, EventArgs e)
        {
            this.mainForm.currentWeb = GetSPWebFromTreeNode(GetSelectedTreeNode());
            this.mainForm.tabControl1.SelectTab("topLevelSitesTabPage");
            this.mainForm.tabControl1.Refresh();
        }

        private void createTopLevelSiteViaWebServicesItem_Click(object sender, EventArgs e)
        {
            this.mainForm.currentWeb = GetSPWebFromTreeNode(GetSelectedTreeNode());
            this.mainForm.tabControl1.SelectTab("topLevelSitesTabPage");
            this.mainForm.tabControl1.Refresh();
        }

        void queryEditQuickLaunchItems_Click(object sender, EventArgs e)
        {
            try
            {
                this.mainForm.currentWeb = GetSPWebFromTreeNode(GetSelectedTreeNode());
                this.mainForm.tabControl1.SelectTab("quickLaunchTabPage");
                this.mainForm.tabControl1.Refresh();

                PopulateQuickLaunchTree(this.mainForm.currentWeb);
            }
            catch (System.Exception Ex)
            {
                string ExceptionMessage = Ex.Message;
            }
        }

        void addTopNavigationItem_Click(object sender, EventArgs e)
        {
            try
            {
                this.mainForm.currentWeb = GetSPWebFromTreeNode(GetSelectedTreeNode());
                this.mainForm.tabControl1.SelectTab("topNavigationTabPage");
                this.mainForm.tabControl1.Refresh();

                PopulateTopNavigationTree(this.mainForm.currentWeb);
            }
            catch (System.Exception Ex)
            {
                string ExceptionMessage = Ex.Message;
            }

            //try
            //{
            //    this.mainForm.currentWeb = GetSPWebFromTreeNode(GetSelectedTreeNode());
            //    SPNavigationNodeCollection topNavigationBarNodes = this.mainForm.currentWeb.Navigation.TopNavigationBar;

            //    SPNavigationNode headerMenuItem = new SPNavigationNode("SharePoint Sites", "", true);
            //    topNavigationBarNodes.AddAsFirst(headerMenuItem);

            //    SPNavigationNode headerMenuItem2 = new SPNavigationNode("Administration", "", false);
            //    topNavigationBarNodes.AddAsFirst(headerMenuItem2);

            //    SPNavigationNode externalSubMenuItem1 = new SPNavigationNode("SharePoint Experts", "http://www.SharePointExperts.com", true);
            //    topNavigationBarNodes[1].Children.AddAsFirst(externalSubMenuItem1);
            //    SPNavigationNode externalSubMenuItem2 = new SPNavigationNode("SharePoint University", "http://www.SharePointU.com", true);
            //    topNavigationBarNodes[1].Children.AddAsFirst(externalSubMenuItem2);

            //    SPNavigationNode internalSubMenuItem1 = new SPNavigationNode("Site Settings", "_layouts/settings.aspx", false);
            //    topNavigationBarNodes[0].Children.AddAsFirst(internalSubMenuItem1);
            //    SPNavigationNode internalSubMenuItem2 = new SPNavigationNode("Create", "_layouts/create.aspx", false);
            //    topNavigationBarNodes[0].Children.AddAsFirst(internalSubMenuItem2);

            //    this.mainForm.currentWeb.Update();
            //}
            //catch (System.Exception Ex)
            //{
            //    string ExceptionMessage = Ex.Message;
            //    int i = 1;
            //}
        }

        void deleteSiteItem_Click(object sender, EventArgs e)
        {
            this.mainForm.currentWeb = GetSPWebFromTreeNode(GetSelectedTreeNode());

            DialogResult result;

            result = MessageBox.Show("Are you sure you want to delete the " + this.mainForm.currentWeb.Title + " site?",
                "Site Deletion Confirmation", MessageBoxButtons.OKCancel);
            if (result == DialogResult.OK)
            {
                this.mainForm.DeleteSite();
            }
        }

        #endregion

        #region Utility Methods

        public TreeNode GetSelectedTreeNode()
        {
            return this.mainForm.sitesTree.SelectedNode;
        }

        public TreeNode GetSelectedQuickLaunchTreeNode()
        {
            return this.mainForm.quickLaunchNodesTreeView.SelectedNode;
        }

        public TreeNode GetSelectedTopNavigationTreeNode()
        {
            return this.mainForm.topNavigationNodesTreeView.SelectedNode;
        }

        public SPWeb GetSPWebFromTreeNode(TreeNode node)
        {
            SPSite site = new SPSite(GetURLFromNodeTag(node.Tag.ToString()));
            SPWeb web = site.OpenWeb();
            this.mainForm.currentWeb = web;
            return web;
        }

        public SPWeb GetSPWebFromQuickLaunchTreeNode(TreeNode node)
        {
            try
            {
                string serverName = "http://" + this.mainForm.sharePointServerNameTextBox.Text;
                SPSite site = new SPSite(serverName + GetURLFromNodeTag(node.Tag.ToString()));
                SPWeb web = site.OpenWeb();
                this.mainForm.currentWeb = web;
                return web;
            }
            catch
            {
                return null;
            }
        }

        public SPSite GetSPSiteFromTreeNode(TreeNode node)
        {
            SPSite site = new SPSite(GetURLFromNodeTag(node.Tag.ToString()));
            return site;
        }

        public string GetGuidFromNodeTag(string tag)
        {
            int startOfSeperatorIndex;
            startOfSeperatorIndex = tag.IndexOf("~~");
            return tag.Substring(0, startOfSeperatorIndex);
        }

        public SPNavigationNode GetSPQuickLaunchNavigationNodeFromTreeNode(SPWeb currentWeb, TreeNode treeNode)
        {
            SPNavigationNodeCollection nodes = currentWeb.Navigation.QuickLaunch;

            returnNode = null;

            return FindMatchingSPNavigationNode(nodes, treeNode);
        }

        public SPNavigationNode GetSPTopNavigationNodeFromTreeNode(SPWeb currentWeb, TreeNode treeNode)
        {
            SPNavigationNodeCollection nodes = currentWeb.Navigation.TopNavigationBar;

            returnNode = null;

            return FindMatchingSPNavigationNode(nodes, treeNode);
        }

        public SPNavigationNode FindMatchingSPNavigationNode(SPNavigationNodeCollection nodes, TreeNode treeNode)
        {
            try
            {
                foreach (SPNavigationNode node in nodes)
                {
                    if (node.Id.ToString() == GetGuidFromNodeTag(treeNode.Tag.ToString()))
                    {
                        returnNode = node;
                        break;
                    }

                    if (node.Children.Count > 0)
                    {
                        if (returnNode == null)
                        {
                            FindMatchingSPNavigationNode(node.Children, treeNode);
                        }
                    }
                }
            }
            catch
            {
            }
            return returnNode;

        }

        public string GetURLFromNodeTag(string tag)
        {
            int startOfSeperatorIndex;
            startOfSeperatorIndex = tag.IndexOf("~~");
            return tag.Substring(startOfSeperatorIndex + 2);
        }

        #endregion
    }
}
